package com.yonge.cooleshow.biz.dal.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Sets;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.auth.config.CustomerServiceConfig;
import com.yonge.cooleshow.biz.dal.config.RongCloudConfig;
import com.yonge.cooleshow.biz.dal.dao.ImUserFriendDao;
import com.yonge.cooleshow.biz.dal.dao.TeacherDao;
import com.yonge.cooleshow.biz.dal.dto.BasicUserInfo;
import com.yonge.cooleshow.biz.dal.entity.ImUserFriend;
import com.yonge.cooleshow.biz.dal.enums.MK;
import com.yonge.cooleshow.biz.dal.service.ImUserFriendService;
import io.rong.messages.TxtMessage;
import io.rong.models.message.PrivateMessage;
import io.rong.models.message.PushExt;
import io.rong.models.response.ResponseResult;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 用户通讯录表(ImUserFriend)表服务实现类
 *
 * @author zx
 * @since 2022-03-22 10:46:00
 */
@Slf4j
@Service("imUserFriendService")
public class ImUserFriendServiceImpl extends ServiceImpl<ImUserFriendDao, ImUserFriend> implements ImUserFriendService {

    @Resource
    private TeacherDao teacherDao;
    @Resource
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private CustomerServiceConfig customerServiceConfig;

    @Override
    public ImUserFriendDao getDao() {
        return this.baseMapper;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveUserFriend(Long teacherId, Set<Long> studentIds) {
        Date now = new Date();
        //老师添加学员联系人
        List<ImUserFriend> imUserFriends = new ArrayList<>();
        BasicUserInfo basicUserInfo = teacherDao.getBasicUserInfo(teacherId);
        List<BasicUserInfo> basicUserInfoList = teacherDao.findBasicUserInfo(studentIds);
        basicUserInfoList.add(basicUserInfo);
        Map<Long, List<BasicUserInfo>> basicUserInfoMap = basicUserInfoList.stream().collect(Collectors.groupingBy(BasicUserInfo::getUserId));
        for(Long studentId : studentIds){
            //老师关联学员
            ImUserFriend teacherFriend = new ImUserFriend();
            //学员关联老师
            ImUserFriend studentFriend = new ImUserFriend();
            List<BasicUserInfo> basicUserInfos = basicUserInfoMap.get(studentId);
            if(CollectionUtils.isNotEmpty(basicUserInfos)){
                BasicUserInfo info = basicUserInfos.get(0);
                teacherFriend.setFriendAvatar(info.getAvatar());
                teacherFriend.setFriendNickname(info.getUsername());
                studentFriend.setFriendAvatar(basicUserInfo.getAvatar());
                studentFriend.setFriendNickname(basicUserInfo.getUsername());
            }
            teacherFriend.setFriendId(studentId);
            teacherFriend.setUserId(teacherId);
            teacherFriend.setCreateTime(now);
            teacherFriend.setUpdateTime(now);

            studentFriend.setFriendId(teacherId);
            studentFriend.setUserId(studentId);
            studentFriend.setCreateTime(now);
            studentFriend.setUpdateTime(now);
            imUserFriends.add(teacherFriend);
            imUserFriends.add(studentFriend);
        }
        baseMapper.insertBatch(imUserFriends);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ImUserFriend getDetail(Long userId) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        BasicUserInfo basicUserInfo = teacherDao.getBasicUserInfo(userId);
        if(sysUser != null && sysUser.getId() != null){
            ImUserFriend imUserFriend = baseMapper.findByUserIdAndFriendId(sysUser.getId(),userId);
            if(Objects.nonNull(imUserFriend)){
                //更新当前用户关联的该用户的详情信息
                imUserFriend.setFriendAvatar(basicUserInfo.getAvatar());
                imUserFriend.setFriendNickname(basicUserInfo.getUsername());
                imUserFriend.setUpdateTime(new Date());
                baseMapper.updateById(imUserFriend);
                return imUserFriend;
            }
        }
        ImUserFriend imUserFriend = new ImUserFriend();
        imUserFriend.setFriendAvatar(basicUserInfo.getAvatar());
        imUserFriend.setFriendNickname(basicUserInfo.getUsername());
        imUserFriend.setFriendId(userId);
        return imUserFriend;
    }

    /**
     * 新用户自动添加客服
     *
     * @param userId    新用户ID
     * @param friendIds 好友ID
     * @return Integer
     */
    @Override
    public Integer registerUserBindCustomerService(Long userId, List<Long> friendIds) {

        // 添加新用户好友
        saveUserFriend(userId, Sets.newHashSet(friendIds));

        try {

            // 设置默认值
            String customerMessage = customerServiceConfig.getCustomerMessage();
            if (StringUtils.isEmpty(customerMessage)) {
                customerMessage = MK.IM_SYS_FRIEND;
            }

            String customerTitle = customerServiceConfig.getCustomerTitle();
            if (StringUtils.isEmpty(customerTitle)) {
                customerTitle = MK.IM_SYS_TITLE;
            }

            // 发送消息
            TxtMessage txtMessage = new TxtMessage(customerMessage, "");

            // 拓展消息
            PushExt pushExt = PushExt.build(customerTitle, 1,
                    new PushExt.HW("channelId", "NORMAL"), new PushExt.VIVO("1"),
                    new PushExt.APNs("", ""),
                    new PushExt.OPPO(""));

            // 发送用户IM通知消息
            PrivateMessage privateMessage = new PrivateMessage()
                    .setSenderId(String.valueOf(friendIds.get(0)))
                    .setTargetId(new String[]{String.valueOf(userId)})
                    .setObjectName(txtMessage.getType())
                    .setContent(txtMessage)
                    .setPushExt(pushExt)
                    .setIsIncludeSender(1);

            ResponseResult privateResult = RongCloudConfig.rongCloud.message.msgPrivate.send(privateMessage);
            log.info("registerUserBindCustomerService userId={}, ret={}", userId, privateResult.getCode());
        } catch (Exception e) {
            log.error("registerUserBindCustomerService userId={}", userId, e);
        }

        return CollectionUtils.size(friendIds);
    }

}

